import { singleton } from 'decorators/singleton'
import logger from 'logger/logger'
import { ConfirmQueue } from 'queue/confirm.queue'
import Web3 from 'web3'
import { TransactionReceipt, AddedAccount } from 'web3-core'
import { ERC20Reactor } from './ERC20Reactor'
import { ERC721Reactor } from './ERC721Reactor'

@singleton
export class BlockChain {
  private web3: Web3
  instanceCacheMap: Map<string, any>
  private accountMaster: AddedAccount
  public erc20Reactor: ERC20Reactor
  public erc721Reactor: ERC721Reactor
  public confirmQueue: ConfirmQueue
  public currentBlockNum: number = 0

  constructor() {
    this.web3 = new Web3(process.env.CHAIN_RPC_URL)
    this.confirmQueue = new ConfirmQueue(this.web3)
    let key = process.env.CHAIN_MASTER_KEY
    this.accountMaster = this.web3.eth.accounts.wallet.add(key)
    this.web3.eth.accounts.wallet.add(process.env.CHAIN_MINT_KEY)
    this.web3.eth.accounts.wallet.add(process.env.CHAIN_ADMIN_KEY)
    this.instanceCacheMap = new Map()
    this.erc20Reactor = new ERC20Reactor(this.web3, process.env.CHAIN_FT_ADDRESS, process.env.CHAIN_FTLOCKER_ADDRESS)
    this.erc721Reactor = new ERC721Reactor(
      this.web3,
      process.env.CHAIN_NFT_ADDRESS,
      process.env.CHAIN_NFTLOCKER_ADDRESS,
    )
    // this.web3.eth.subscribe('newBlockHeaders', (error, event) => {
    //   if (!error) {
    //     console.log(event.number)
    //     return
    //   }
    //   console.log(error)
    // })
  }

  public async getContractInstance(address: string, abi: any) {
    if (!this.instanceCacheMap.has(address)) {
      const instance = new this.web3.eth.Contract(abi, address, { from: this.accountMaster.address })
      this.instanceCacheMap.set(address, instance)
    }
    return this.instanceCacheMap.get(address)
  }

  public async getTransactionReceipt(txHash: string) {
    return this.web3.eth.getTransactionReceipt(txHash)
  }

  public async getTxConfirms(txhash: string) {
    const receipt: TransactionReceipt = await this.getTransactionReceipt(txhash)
    const latest = await this.web3.eth.getBlockNumber()
    return latest - receipt.blockNumber + 1
  }

  public async updateCurrenBlockNum() {
    this.currentBlockNum = await this.web3.eth.getBlockNumber()
    logger.debug(`update block num: ${this.currentBlockNum}`)
  }
}
