import { FastifyPluginAsync, FastifyReply, FastifyRequest } from 'fastify'
import fastifyPlugin from 'fastify-plugin'

declare module 'fastify' {
  interface FastifyRequest {
    roles?: string[]
    user?: any
    token?: string
  }
  interface FastifyInstance {
    apiAuth: (request: FastifyRequest, reply: FastifyReply) => {}
  }
}

export interface ApiAuthOptions {
  secret: string
  expiresIn: string
}

const apiAuthPlugin: FastifyPluginAsync<ApiAuthOptions> = async function (fastify, opts) {
  fastify.register(require('fastify-jwt'), {
    secret: opts.secret,
    sign: { expiresIn: opts.expiresIn },
  })
  // 只有路由配置的role为anon才不需要过滤
  fastify.decorate('apiAuth', async function (request: FastifyRequest, reply: FastifyReply) {
    if (!request.roles || request.roles.indexOf('anon') == -1) {
      try {
        if (!request.token) {
          return reply.send({ code: 11, msg: 'need login' })
        }
        //@ts-ignore
        const data = this.jwt.verify(request.token)
        if (!data || !data.id) {
          return reply.send({ code: 10, msg: 'need login' })
        }
        // let account = await Account.findById(data.id)
        // if (!account) {
        //   return reply.send({ code: 10, msg: 'need login' })
        // }
        // request.user = account
      } catch (err) {
        return reply.send({ code: 401, msg: 'need auth' })
      }
    }
  })
}

export default fastifyPlugin(apiAuthPlugin, '3.x')
